% Example_9_3
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
close all, clear all

% Kernel filter design
fk = [0,50/200,100/200,1]; mk=[1,1,0,0];            % Setting the design parameters
w = [1,3.333];                                                     % Passband/stopband weighting coefficients
hk = firpm(24,fk,mk,w);                                      % Computation of filter coefficients

[Hk,fk]=freqz(hk,1,512,400);

% Decimation/ interpolation filter design
f0 = [0,50/1000,350/1000,450/1000,750/1000,850/1000];   % Setting the design parameters
m0 = [1,1,0,0,0,0];                                                                % Setting the design parameters
w0 = [w,w(2)];                                                                      % Passband/stopband weighting coefficients
hd = firpm(18,f0,m0,w0);                                                      % Computation of filter coefficients

[Hd,f]=freqz(hd,1,512,2000);

% Computation of the multistage filter characteristic
hkm=zeros(size(1:5*length(hk)));
hkm([1:5:length(hkm)])=hk;
[Hkm,f]=freqz(hkm,1,512,2000);
H=(Hd.*Hkm).*Hd;

figure (1)
subplot(2,1,1)
plot(fk,20*log10(abs(Hk)));
xlabel('Frequency [Hz]'),ylabel('Gain [dB]')
%title('(a) Kernel filter')
axis([0,200,-80,5]), grid
subplot(2,1,2)
plot(f,20*log10(abs(Hd)));
xlabel('Frequency [Hz]'),ylabel('Gain [dB]')
%title('(b) Decimation and interpolation filter')
axis([0,1000,-80,5]), grid

figure (2)
subplot(2,1,1)
plot(f,20*log10(abs(H)));
xlabel('Frequency [Hz]'),ylabel('Gain [dB]')
%title('(c) Multistage filter')
axis([0,1000,-90,5]), grid
h=hd;
n=0:length(h)-1;
Fx=2000; % input sampling frequency
M=5; % decimation factor
Fy=Fx/M; % output sampling frequency
for r=1:1001
    F(r)=(r-1)*Fx/400;
omega=2*pi*F(r)/Fx;
W=exp(-i*n*omega);
H(r)=sum(h.*W); % FIR filter frequency response
end

figure (2)
subplot(2,1,2)
% Computation of aliasing in the frequency band of the low-rate signal.
% For k=0 program computes unaliased component.
% For k=1,2,3,4 program computes the aliased components.
k=0;
for l=1:5
for r=1:201
    FF(r)=(r-1)*Fy/75;
omega=2*pi*FF(r)/Fy;
W2=exp(-i*n*(omega-2*k*pi)/M);
Hal(r)=sum(h.*W2);
end
k=k+1;
plot(FF,20*log10(abs(Hal)))
hold on
end
xlabel('Frequency [Hz]'),ylabel('Gain [dB]')
axis([0,75,-90,5]), grid on
hold off